<?php

namespace App\Http\Livewire\Backend;

use App\Domains\Auth\Models\User;
use App\Domains\Product\Models\Product;
use Illuminate\Database\Eloquent\Builder;
use Rappasoft\LaravelLivewireTables\DataTableComponent;
use Rappasoft\LaravelLivewireTables\Views\Column;
use Rappasoft\LaravelLivewireTables\Views\Filter;

/**
 * Class UsersTable.
 */
class ProductsTable extends DataTableComponent
{
    /**
     * @var
     */
    public $status;
    public $user;

    /**
     * @var array|string[]
     */
    public array $sortNames = [
        'Price' => 'Price'
    ];

    /**
     * @var array|string[]
     */
    public array $filterNames = [
        'stage' => 'Stage'
    ];

    /**
     * @param  string  $status
     */
    public function mount($status = true, $user = null): void
    {
        $this->status = $status;
        $this->user = $user;
    }

    /**
     * @return Builder
     */
    public function query(): Builder
    {
        if($this->user)
            $query = Product::where('user_id', $this->user->id)->with('brand', 'type', 'photos')->latest();
        else
            $query = Product::with('brand', 'type', 'photos')->oldest();

        if ($this->status === Product::STAGE_CREATED) {
            $query = $query->byStage($this->status);
        }elseif ($this->status === Product::STAGE_PRE_CHECK) {
            $query = $query->byStage($this->status);
        } elseif ($this->status === Product::STAGE_ARRIVAL_CHECK) {
            $query = $query->byStage(Product::STAGE_ARRIVAL_CHECK);
        } elseif ($this->status === Product::STAGE_UPLOADED) {
            $query = $query->byStage(Product::STAGE_UPLOADED);
        } elseif ($this->status === "deleted") {
            $query = $query->onlyTrashed();
        }

        return $query
            ->when($this->getFilter('search'), fn ($query, $term) => $query->search($term))
            ->when($this->getFilter('stage'), fn ($query, $stage) => $query->where('stage', $stage));
    }

    /**
     * @return array
     */
    public function filters(): array
    {
        return [
            'stage' => Filter::make('Product Stage')
                ->select([
                    '' => 'Any',
                    Product::STAGE_CREATED => 'Created',
                    Product::STAGE_PRE_CHECK => 'Pre-Checks',
                    Product::STAGE_ARRIVAL_CHECK => 'Arrival Checks',
                    Product::STAGE_UPLOADED => 'Uploaded',
                ]),
        ];
    }

    /**
     * @return array
     */
    public function columns(): array
    {
        return [
            Column::make(__('Unique Tag'), 'unique_tag')
                ->sortable(),

            Column::make(__('Type'), 'type_id')
                ->sortable(),

            Column::make(__('Damage Preset'), 'damage_preset')
                ->sortable(),

            Column::make(__('Size on Label'), 'size_on_label')
                ->sortable(),

            Column::make(__('Brand'), 'brand_id')
                ->sortable(),

            Column::make(__('Price'), 'price')
                ->sortable(),

            Column::make(__('Embroidered/Printed'), 'embroidered_or_printed')
                ->sortable(),

            Column::make(__('Stage'), 'stage')
                ->sortable(),

            Column::make(__('Actions')),
        ];
    }

    /**
     * @return string
     */
    public function rowView(): string
    {
        return 'backend.product.includes.row';
    }
}
